@@ -5,19 +5,24 @@
 """
 import os
 import subprocess
+from contextlib import closing
 
 from bcbio.broad import picardrun
 
 class BroadRunner:
     """Simplify running Broad commandline tools.
     """
-    def __init__(self, picard_dir, gatk_dir="", max_memory=None):
+    def __init__(self, picard_dir, gatk_dir="", max_memory=None,
+                 config=None):
         self._memory_args = []
         if not max_memory:
             max_memory = "6g"
         self._memory_args.append("-Xmx%s" % max_memory)
         self._picard_dir = picard_dir
         self._gatk_dir = gatk_dir or picard_dir
+        if config is None:
+            config = {}
+        self._config = config
 
     def run_fn(self, name, *args, **kwds):
         """Run pre-built functionality that used Broad tools by name.
@@ -45,8 +50,20 @@ class BroadRunner:
         subprocess.check_call(cl)
 
     def run_gatk(self, params, tmp_dir=None):
-        gatk_jar = self._get_jar("GenomeAnalysisTK")
+        support_parallel = ["UnifiedGenotyper", "CountCovariates", "VariantEval",
+                            "VariantRecalibrator"]
+        gatk_jar = self._get_jar("GenomeAnalysisTK", ["GenomeAnalysisTKLite"])
         local_args = []
+        cores = self._config.get("resources", {}).get("gatk", {}).get("cores", None)
+        if cores:
+            do_parallel = False
+            for check in support_parallel:
+                if check in params:
+                    do_parallel = True
+            if do_parallel:
+                params.extend(["-nt", str(cores)])
+        if len([x for x in params if x.startswith(("-U", "--unsafe"))]) == 0:
+            params.extend(["-U", "LENIENT_VCF_PROCESSING"])
         if tmp_dir:
             local_args.append("-Djava.io.tmpdir=%s" % tmp_dir)
         cl = ["java"] + self._memory_args + local_args + \
@@ -54,7 +71,18 @@ class BroadRunner:
         #print " ".join(cl)
         subprocess.check_call(cl)
 
-    def _get_jar(self, command):
+    def has_gatk_full(self):
+        """Check if we have the full GATK jar, including non-open source parts.
+        """
+        gatk_jar = self._get_jar("GenomeAnalysisTK", ["GenomeAnalysisTKLite"])
+        cl = ["java", "-jar", gatk_jar, "-h"]
+        with closing(subprocess.Popen(cl, stdout=subprocess.PIPE).stdout) as stdout:
+            for line in stdout:
+                if line.strip().startswith("HaplotypeCaller"):
+                    return True
+        return False
+
+    def _get_jar(self, command, alts=None):
         """Retrieve the jar for running the specified command.
         """
         dirs = []
@@ -65,13 +93,16 @@ class BroadRunner:
                          os.path.join(bdir, "GATK"),
                          os.path.join(bdir, "GATK", "dist"),
                          os.path.join(bdir, "Picard-private", "dist")])
-        for dir_check in dirs:
-            check_file = os.path.join(dir_check, "%s.jar" % command)
-            if os.path.exists(check_file):
-                return check_file
+        if alts is None: alts = []
+        for check_cmd in [command] + alts:
+            for dir_check in dirs:
+                check_file = os.path.join(dir_check, "%s.jar" % check_cmd)
+                if os.path.exists(check_file):
+                    return check_file
         raise ValueError("Could not find jar %s in %s" % (command, self._picard_dir))
 
 def runner_from_config(config):
     return BroadRunner(config["program"]["picard"],
                        config["program"].get("gatk", ""),
-                       max_memory=config["algorithm"].get("java_memory", ""))
+                       max_memory=config["algorithm"].get("java_memory", ""),
+                       config=config)
