@@ -28,13 +28,21 @@ def generate_align_summary(bam_file, is_paired, sam_ref, sample_name,
         return _generate_pdf(graphs, summary, overrep, bam_file, sample_name,
                              dirs, config)
 
+def _safe_latex(to_fix):
+    """Escape characters that make LaTeX unhappy.
+    """
+    chars = ["%", "_", "&", "#"]
+    for char in chars:
+        to_fix = to_fix.replace(char, "\\%s" % char)
+    return to_fix
+
 def _generate_pdf(graphs, summary, overrep, bam_file, sample_name,
                   dirs, config):
     base = os.path.splitext(os.path.basename(bam_file))[0]
     sample_name = base if sample_name is None else " : ".join(sample_name)
     tmpl = Template(_section_template)
-    sample_name = "%s (%s)" % (sample_name.replace("_", "\_"),
-                               base.replace("_", "\_"))
+    sample_name = "%s (%s)" % (_safe_latex(sample_name),
+                               _safe_latex(base))
     recal_plots = sorted(glob.glob(os.path.join(dirs["work"], "reports", "images",
                                                 "%s*-plot.pdf" % base)))
     section = tmpl.render(name=sample_name, summary=None,
@@ -46,8 +54,9 @@ def _generate_pdf(graphs, summary, overrep, bam_file, sample_name,
     out_tmpl = Template(_base_template)
     with open(out_file, "w") as out_handle:
         out_handle.write(out_tmpl.render(parts=[section]))
-    cl = [config.get("program", {}).get("pdflatex", "pdflatex"), out_file]
-    subprocess.check_call(cl)
+    if config["algorithm"].get("write_summary", True):
+        cl = [config.get("program", {}).get("pdflatex", "pdflatex"), out_file]
+        subprocess.check_call(cl)
     return "%s.pdf" % os.path.splitext(out_file)[0]
 
 def _graphs_and_summary(bam_file, sam_ref, is_paired, tmp_dir, config):
@@ -77,6 +86,58 @@ def _update_summary_table(summary_table, ref_file, fastqc_stats):
         ref_org.replace("_", " "), ""))
     return summary_table
 
+# ## Generate project level QC summary for quickly assessing large projects
+
+def write_project_summary(samples):
+    """Write project summary information on the provided samples.
+    """
+    def _nocommas(x):
+        return x.replace(",", "")
+    def _percent(x):
+        return x.replace("(", "").replace(")", "").replace("\\", "")
+    out_file = os.path.join(samples[0][0]["dirs"]["work"], "project-summary.csv")
+    sample_info = _get_sample_summaries(samples)
+    header = ["Total", "Aligned", "Pair duplicates", "Insert size",
+              "On target bases", "Mean target coverage", "10x coverage targets",
+              "Zero coverage targets", "Total variations", "In dbSNP",
+              "Transition/Transversion (all)", "Transition/Transversion (dbSNP)",
+              "Transition/Transversion (novel)"]
+    select = [(0, _nocommas), (1, _percent), (1, _percent), (0, None),
+              (1, _percent), (0, None), (0, _percent),
+              (0, _percent), (0, None), (0, _percent),
+              (0, None), (0, None), (0, None)]
+    rows = [["Sample"] + header]
+    for name, info in sample_info:
+        cur = [name]
+        for col, (i, prep_fn) in zip(header, select):
+            val = info.get(col, ["", ""])[i]
+            if prep_fn and val:
+                val = prep_fn(val)
+            cur.append(val)
+        rows.append(cur)
+    with open(out_file, "w") as out_handle:
+        writer = csv.writer(out_handle)
+        for row in rows:
+            writer.writerow(row)
+
+def _get_sample_summaries(samples):
+    """Retrieve high level summary information for each sample.
+    """
+    out = []
+    with utils.curdir_tmpdir() as tmp_dir:
+        for sample in (x[0] for x in samples):
+            is_paired = sample.get("fastq2", None) not in ["", None]
+            _, summary, _ = _graphs_and_summary(sample["work_bam"], sample["sam_ref"],
+                                            is_paired, tmp_dir, sample["config"])
+            sample_info = {}
+            for xs in summary:
+                n = xs[0]
+                if n is not None:
+                    sample_info[n] = xs[1:]
+            sample_name = ";".join([x for x in sample["name"] if x])
+            out.append((sample_name, sample_info))
+    return out
+
 # ## Run and parse read information from FastQC
 
 def fastqc_report(bam_file, config):
@@ -109,11 +170,11 @@ class FastQCParser:
     def get_fastqc_summary(self):
         stats = {}
         for stat_line in self._fastqc_data_section("Basic Statistics")[1:]:
-            k, v = [self._safe_latex(x) for x in stat_line.split("\t")[:2]]
+            k, v = [_safe_latex(x) for x in stat_line.split("\t")[:2]]
             stats[k] = v
         over_rep = []
         for line in self._fastqc_data_section("Overrepresented sequences")[1:]:
-            parts = [self._safe_latex(x) for x in line.split("\t")]
+            parts = [_safe_latex(x) for x in line.split("\t")]
             over_rep.append(parts)
             over_rep[-1][0] = self._splitseq(over_rep[-1][0])
         return stats, over_rep[:self._max_overrep]
@@ -144,13 +205,6 @@ class FastQCParser:
                         out.append(line.rstrip("\r\n"))
         return out
 
-    def _safe_latex(self, to_fix):
-        """Escape characters that make LaTeX unhappy.
-        """
-        chars = ["%", "_", "&"]
-        for char in chars:
-            to_fix = to_fix.replace(char, "\\%s" % char)
-        return to_fix
 
 def _run_fastqc(bam_file, config):
     out_base = "fastqc"
@@ -195,7 +249,8 @@ def summary_metrics(run_info, analysis_dir, fc_name, fc_date, fastq_dir):
     tab_out = []
     lane_info = []
     sample_info = []
-    for run in run_info["details"]:
+    for lane_xs in run_info["details"]:
+        run = lane_xs[0]
         tab_out.append([run["lane"], run.get("researcher", ""),
             run.get("name", ""), run.get("description")])
         base_info = dict(
@@ -207,17 +262,16 @@ def summary_metrics(run_info, analysis_dir, fc_name, fc_date, fastq_dir):
         cur_lane_info["metrics"] = _bustard_stats(run["lane"], fastq_dir,
                                                   fc_date, analysis_dir)
         lane_info.append(cur_lane_info)
-        for barcode in run.get("multiplex", [None]):
+        for lane_x in lane_xs:
             cur_name = "%s_%s_%s" % (run["lane"], fc_date, fc_name)
-            if barcode:
-                cur_name = "%s_%s-" % (cur_name, barcode["barcode_id"])
+            if lane_x["barcode_id"]:
+                cur_name = "%s_%s-" % (cur_name, lane_x["barcode_id"])
             stats = _metrics_from_stats(_lane_stats(cur_name, analysis_dir))
             if stats:
                 cur_run_info = copy.deepcopy(base_info)
                 cur_run_info["metrics"] = stats
-                cur_run_info["barcode_id"] = str(barcode["barcode_id"]) if barcode else ""
-                cur_run_info["barcode_type"] = (str(barcode.get("barcode_type", ""))
-                                                if barcode else "")
+                cur_run_info["barcode_id"] = str(lane_x["barcode_id"])
+                cur_run_info["barcode_type"] = str(lane_x.get("barcode_type", ""))
                 sample_info.append(cur_run_info)
     return lane_info, sample_info, tab_out
 
@@ -233,7 +287,7 @@ def _metrics_from_stats(stats):
                 )
         metrics = dict()
         for stat_name, metric_name in s_to_m.iteritems():
-            metrics[metric_name] = stats[stat_name]
+            metrics[metric_name] = stats.get(stat_name, 0)
         return metrics
 
 def _bustard_stats(lane_num, fastq_dir, fc_date, analysis_dir):
