@@ -27,79 +27,56 @@ import yaml
 
 from bcbio.solexa.flowcell import get_fastq_dir
 from bcbio import utils
-from bcbio.log import create_log_handler
-from bcbio.distributed import messaging
+from bcbio.log import logger, setup_logging
+from bcbio.distributed.messaging import parallel_runner
 from bcbio.pipeline.run_info import get_run_info
-from bcbio.pipeline import log
 from bcbio.pipeline.demultiplex import add_multiplex_across_lanes
 from bcbio.pipeline.merge import organize_samples
-from bcbio.pipeline.qcsummary import write_metrics
-from bcbio.pipeline import sample
-from bcbio.pipeline import lane
+from bcbio.pipeline.qcsummary import write_metrics, write_project_summary
+from bcbio.variation.realign import parallel_realign_sample
+from bcbio.variation.genotype import parallel_variantcall, combine_multiple_callers
+from bcbio.pipeline.config_loader import load_config
 
-def main(config_file, fc_dir, run_info_yaml=None):
-    with open(config_file) as in_handle:
-        config = yaml.load(in_handle)
-    log_handler = create_log_handler(config, log.name)
-    with log_handler.applicationbound():
-        run_main(config, config_file, fc_dir, run_info_yaml)
-
-def run_main(config, config_file, fc_dir, run_info_yaml):
+def main(config_file, fc_dir, run_info_yaml=None, num_cores=None):
+    config = load_config(config_file)
     work_dir = os.getcwd()
-    align_dir = os.path.join(work_dir, "alignments")
+    if config.get("log_dir", None) is None:
+        config["log_dir"] = os.path.join(work_dir, "log")
+    if num_cores:
+        config["algorithm"]["num_cores"] = int(num_cores)
+    setup_logging(config)
+    run_main(config, config_file, fc_dir, work_dir, run_info_yaml)
 
+def run_main(config, config_file, fc_dir, work_dir, run_info_yaml):
+    align_dir = os.path.join(work_dir, "alignments")
+    run_module = "bcbio.distributed"
     fc_name, fc_date, run_info = get_run_info(fc_dir, config, run_info_yaml)
     fastq_dir, galaxy_dir, config_dir = _get_full_paths(get_fastq_dir(fc_dir),
                                                         config, config_file)
     config_file = os.path.join(config_dir, os.path.basename(config_file))
     dirs = {"fastq": fastq_dir, "galaxy": galaxy_dir, "align": align_dir,
             "work": work_dir, "flowcell": fc_dir, "config": config_dir}
-    run_items = add_multiplex_across_lanes(run_info["details"], dirs["fastq"], fc_name)
+    run_parallel = parallel_runner(run_module, dirs, config, config_file)
 
     # process each flowcell lane
+    run_items = add_multiplex_across_lanes(run_info["details"], dirs["fastq"], fc_name)
     lanes = ((info, fc_name, fc_date, dirs, config) for info in run_items)
-    lane_items = _run_parallel("process_lane", lanes, dirs, config, config_file)
-    align_items = _run_parallel("process_alignment", lane_items, dirs, config,
-                                config_file)
+    lane_items = run_parallel("process_lane", lanes)
+    align_items = run_parallel("process_alignment", lane_items)
     # process samples, potentially multiplexed across multiple lanes
-    sample_files, sample_fastq, sample_info = \
-                  organize_samples(dirs, fc_name, fc_date, run_items)
-    samples = ((n, sample_fastq[n], sample_info[n], bam_files, dirs, config, config_file)
-               for n, bam_files in sample_files)
-    sample_items = _run_parallel("process_sample", samples, dirs, config, config_file)
-
+    samples = organize_samples(align_items, dirs, config_file)
+    samples = run_parallel("merge_sample", samples)
+    samples = run_parallel("recalibrate_sample", samples)
+    samples = parallel_realign_sample(samples, run_parallel)
+    samples = parallel_variantcall(samples, run_parallel)
+    samples = run_parallel("postprocess_variants", samples)
+    samples = combine_multiple_callers(samples)
+    samples = run_parallel("detect_sv", samples)
+    run_parallel("process_sample", samples)
+    run_parallel("generate_bigwig", samples, {"programs": ["ucsc_bigwig"]})
+    write_project_summary(samples)
     write_metrics(run_info, fc_name, fc_date, dirs)
 
-def _run_parallel(fn_name, items, dirs, config, config_file):
-    """Process a supplied function: single, multi-processor or distributed.
-    """
-    parallel = config["algorithm"]["num_cores"]
-    if str(parallel).lower() == "messaging":
-        runner = messaging.runner(dirs, config, config_file)
-        return runner(fn_name, items)
-    else:
-        out = []
-        fn = globals()[fn_name]
-        with utils.cpmap(int(parallel)) as cpmap:
-            for data in cpmap(fn, items):
-                if data:
-                    out.extend(data)
-        return out
-
-# ## multiprocessing ready entry points
-
-@utils.map_wrap
-def process_lane(*args):
-    return lane.process_lane(*args)
-
-@utils.map_wrap
-def process_alignment(*args):
-    return lane.process_alignment(*args)
-
-@utils.map_wrap
-def process_sample(*args):
-    return sample.process_sample(*args)
-
 # ## Utility functions
 
 def _get_full_paths(fastq_dir, config, config_file):
@@ -112,10 +89,11 @@ def _get_full_paths(fastq_dir, config, config_file):
 
 if __name__ == "__main__":
     parser = OptionParser()
+    parser.add_option("-n", "--numcores", dest="num_cores", default=None)
     (options, args) = parser.parse_args()
     if len(args) < 2:
         print "Incorrect arguments"
         print __doc__
         sys.exit()
-    kwargs = dict()
+    kwargs = {"num_cores": options.num_cores}
     main(*args, **kwargs)
