@@ -34,19 +34,16 @@ import yaml
 import logbook
 
 from bcbio.solexa import samplesheet
-from bcbio.log import create_log_handler
+from bcbio.log import create_log_handler, logger2
 from bcbio import utils
 from bcbio.distributed import messaging
 from bcbio.solexa.flowcell import (get_flowcell_info, get_fastq_dir, get_qseq_dir)
-
-LOG_NAME = os.path.splitext(os.path.basename(__file__))[0]
-log = logbook.Logger(LOG_NAME)
+from bcbio.pipeline.config_loader import load_config
 
 def main(local_config, post_config_file=None,
          process_msg=True, store_msg=True, qseq=True, fastq=True):
-    with open(local_config) as in_handle:
-        config = yaml.load(in_handle)
-    log_handler = create_log_handler(config, LOG_NAME)
+    config = load_config(local_config)
+    log_handler = create_log_handler(config)
 
     with log_handler.applicationbound():
         search_for_new(config, local_config, post_config_file,
@@ -63,15 +60,15 @@ def search_for_new(config, config_file, post_config_file,
                 # Injects run_name on logging calls.
                 # Convenient for run_name on "Subject" for email notifications
                 with logbook.Processor(lambda record: record.extra.__setitem__('run', os.path.basename(dname))):
-                    log.info("The instrument has finished dumping on directory %s" % dname)
+                    logger2.info("The instrument has finished dumping on directory %s" % dname)
                     _update_reported(config["msg_db"], dname)
                     _process_samplesheets(dname, config)
                     if qseq:
-                        log.info("Generating qseq files for %s" % dname)
+                        logger2.info("Generating qseq files for %s" % dname)
                         _generate_qseq(get_qseq_dir(dname), config)
                     fastq_dir = None
                     if fastq:
-                        log.info("Generating fastq files for %s" % dname)
+                        logger2.info("Generating fastq files for %s" % dname)
                         fastq_dir = _generate_fastq(dname, config)
                     _post_process_run(dname, config, config_file,
                                       fastq_dir, post_config_file,
@@ -81,15 +78,16 @@ def _post_process_run(dname, config, config_file, fastq_dir, post_config_file,
                       process_msg, store_msg):
     """With a finished directory, send out message or process directly.
     """
+    run_module = "bcbio.distributed.tasks"
     # without a configuration file, send out message for processing
     if post_config_file is None:
         store_files, process_files = _files_to_copy(dname)
         if process_msg:
-            finished_message("analyze_and_upload", dname,
+            finished_message("analyze_and_upload", run_module, dname,
                              process_files, config, config_file)
         if store_msg:
             raise NotImplementedError("Storage server needs update.")
-            finished_message("long_term_storage", dname,
+            finished_message("long_term_storage", run_module, dname,
                              store_files, config, config_file)
     # otherwise process locally
     else:
@@ -99,13 +97,16 @@ def analyze_locally(dname, post_config_file, fastq_dir):
     """Run analysis directly on the local machine.
     """
     assert fastq_dir is not None
-    with open(post_config_file) as in_handle:
-        post_config = yaml.load(in_handle)
-    run_yaml = os.path.join(dname, "run_info.yaml")
+    post_config = load_config(post_config_file)
     analysis_dir = os.path.join(fastq_dir, os.pardir, "analysis")
     utils.safe_makedir(analysis_dir)
     with utils.chdir(analysis_dir):
-        cl = [post_config["analysis"]["process_program"], post_config_file, fastq_dir]
+        if post_config["algorithm"]["num_cores"] == "messaging":
+            prog = post_config["analysis"]["distributed_process_program"]
+        else:
+            prog = post_config["analysis"]["process_program"]
+        cl = [prog, post_config_file, dname]
+        run_yaml = os.path.join(dname, "run_info.yaml")
         if os.path.exists(run_yaml):
             cl.append(run_yaml)
         subprocess.check_call(cl)
@@ -116,7 +117,7 @@ def _process_samplesheets(dname, config):
     ss_file = samplesheet.run_has_samplesheet(dname, config)
     if ss_file:
         out_file = os.path.join(dname, "run_info.yaml")
-        log.info("CSV Samplesheet %s found, converting to %s" %
+        logger2.info("CSV Samplesheet %s found, converting to %s" %
                  (ss_file, out_file))
         samplesheet.csv2yaml(ss_file, out_file)
 
@@ -139,7 +140,7 @@ def _generate_fastq(fc_dir, config):
                   ",".join(lanes)]
             if postprocess_dir:
                 cl += ["-o", fastq_dir]
-            log.debug("Converting qseq to fastq on all lanes.")
+            logger2.debug("Converting qseq to fastq on all lanes.")
             subprocess.check_call(cl)
     return fastq_dir
 
@@ -161,6 +162,7 @@ def _generate_qseq(bc_dir, config):
         (out, _) = p.communicate()
         olb_version = float(out.strip().split()[-1].rsplit(".", 1)[0])
         if olb_version > 1.8:
+            cl += ["-P", ".clocs"]
             cl += ["-b", bc_dir]
         else:
             cl += ["-i", bc_dir, "-p", os.path.split(bc_dir)[0]]
@@ -270,10 +272,11 @@ def _update_reported(msg_db, new_dname):
     with open(msg_db, "a") as out_handle:
         out_handle.write("%s\n" % new_dname)
 
-def finished_message(fn_name, directory, files_to_copy, config, config_file):
+def finished_message(fn_name, run_module, directory, files_to_copy,
+                     config, config_file):
     """Wait for messages with the give tag, passing on to the supplied handler.
     """
-    log.debug("Calling remote function: %s" % fn_name)
+    logger2.debug("Calling remote function: %s" % fn_name)
     user = getpass.getuser()
     hostname = socket.gethostbyaddr(socket.gethostname())[0]
     data = dict(
@@ -285,7 +288,7 @@ def finished_message(fn_name, directory, files_to_copy, config, config_file):
             )
     dirs = {"work": os.getcwd(),
             "config": os.path.dirname(config_file)}
-    runner = messaging.runner(dirs, config, config_file, wait=False)
+    runner = messaging.runner(run_module, dirs, config, config_file, wait=False)
     runner(fn_name, [[data]])
 
 if __name__ == "__main__":
