@@ -29,10 +29,10 @@ from bcbio.solexa.flowcell import get_fastq_dir
 from bcbio.pipeline.run_info import get_run_info
 from bcbio.galaxy.api import GalaxyApiAccess
 from bcbio import utils
+from bcbio.pipeline.config_loader import load_config
 
 def main(config_file, fc_dir, analysis_dir, run_info_yaml=None):
-    with open(config_file) as in_handle:
-        config = yaml.load(in_handle)
+    config = load_config(config_file)
     galaxy_api = (GalaxyApiAccess(config['galaxy_url'], config['galaxy_api_key'])
                   if config.has_key("galaxy_api_key") else None)
     fc_name, fc_date, run_info = get_run_info(fc_dir, config, run_info_yaml)
@@ -40,11 +40,11 @@ def main(config_file, fc_dir, analysis_dir, run_info_yaml=None):
     base_folder_name = "%s_%s" % (fc_date, fc_name)
     run_details = lims_run_details(run_info, base_folder_name)
     for (library_name, access_role, dbkey, lane, bc_id, name, desc,
-            local_name) in run_details:
+            local_name, fname_out) in run_details:
         library_id = (get_galaxy_library(library_name, galaxy_api)
                       if library_name else None)
         upload_files = list(select_upload_files(local_name, bc_id, fc_dir,
-                                                analysis_dir, config))
+                                                analysis_dir, config, fname_out))
         if len(upload_files) > 0:
             print lane, bc_id, name, desc, library_name
             print "Creating storage directory"
@@ -54,7 +54,8 @@ def main(config_file, fc_dir, analysis_dir, run_info_yaml=None):
             else:
                 cur_galaxy_files = []
             store_dir = move_to_storage(lane, bc_id, base_folder_name, upload_files,
-                                        cur_galaxy_files, config, config_file)
+                                        cur_galaxy_files, config, config_file,
+                                        fname_out)
             if store_dir and library_id:
                 print "Uploading directory of files to Galaxy"
                 print galaxy_api.upload_directory(library_id, folder['id'],
@@ -71,29 +72,32 @@ def main(config_file, fc_dir, analysis_dir, run_info_yaml=None):
 def lims_run_details(run_info, base_folder_name):
     """Retrieve run infomation on a flow cell from Next Gen LIMS.
     """
-    for lane_info in (l for l in run_info["details"] if l.has_key("researcher")
-                      or not run_info["run_id"]):
-        if lane_info.get("private_libs", None) is not None:
-            libname, role = _get_galaxy_libname(lane_info["private_libs"],
-                                                lane_info["lab_association"],
-                                                lane_info["researcher"])
-        elif lane_info.has_key("galaxy_library"):
-            libname = lane_info["galaxy_library"]
-            role = lane_info["galaxy_role"]
-        else:
-            libname, role = (None, None)
-        for barcode in lane_info.get("multiplex", [None]):
-            remote_folder = str(lane_info.get("name", lane_info["lane"]))
-            description = ": ".join([lane_info[n] for n in ["researcher", "description"]
-                                     if lane_info.has_key(n)])
-            local_name = "%s_%s" % (lane_info["lane"], base_folder_name)
-            if barcode:
-                remote_folder += "_%s" % barcode["barcode_id"]
-                description += ": %s" % barcode["name"]
-                local_name += "_%s" % barcode["barcode_id"]
-            yield (libname, role, lane_info["genome_build"],
-                    lane_info["lane"], barcode["barcode_id"] if barcode else "",
-                    remote_folder, description, local_name)
+    for lane_items in run_info["details"]:
+        for lane_info in lane_items:
+            if not run_info["run_id"] or lane_info.has_key("researcher"):
+                if lane_info.get("private_libs", None) is not None:
+                    libname, role = _get_galaxy_libname(lane_info["private_libs"],
+                                                        lane_info["lab_association"],
+                                                        lane_info["researcher"])
+                elif lane_info.has_key("galaxy_library"):
+                    libname = lane_info["galaxy_library"]
+                    role = lane_info["galaxy_role"]
+                else:
+                    libname, role = (None, None)
+                remote_folder = str(lane_info.get("name", lane_info["lane"]))
+                description = ": ".join([lane_info[n] for n in ["researcher", "description"]
+                                         if lane_info.has_key(n)])
+                if lane_info.get("description_filenames", False):
+                    fname_out = lane_info["description"]
+                else:
+                    fname_out = None
+                local_name = "%s_%s" % (lane_info["lane"], base_folder_name)
+                if lane_info["barcode_id"] is not None:
+                    remote_folder += "_%s" % lane_info["barcode_id"]
+                    local_name += "_%s" % lane_info["barcode_id"]
+                yield (libname, role, lane_info["genome_build"],
+                       lane_info["lane"], lane_info["barcode_id"],
+                       remote_folder, description, local_name, fname_out)
 
 def _get_galaxy_libname(private_libs, lab_association, researcher):
     """Retrieve most appropriate Galaxy data library.
@@ -121,9 +125,24 @@ def _get_galaxy_libname(private_libs, lab_association, researcher):
         except (IndexError, ValueError):
             return private_libs[0]
 
-def select_upload_files(base, bc_id, fc_dir, analysis_dir, config):
+def select_upload_files(base, bc_id, fc_dir, analysis_dir, config, fname_out=None):
     """Select fastq, bam alignment and summary files for upload to Galaxy.
     """
+    def _name_with_ext(orig_file, ext):
+        """Return a normalized filename without internal processing names.
+
+        Use specific base out filename if specific, allowing configuration
+        named output files.
+        """
+        if fname_out is None:
+            base = os.path.basename(orig_file).split("-")[0]
+        else:
+            base = fname_out
+        for extra in ["_trim"]:
+            if base.endswith(extra):
+                base = base[:-len(extra)]
+        return "%s%s" % (base, ext)
+
     base_glob = _dir_glob(base, analysis_dir)
     # Configurable upload of fastq files -- BAM provide same information, compacted
     if config["algorithm"].get("upload_fastq", True):
@@ -141,24 +160,30 @@ def select_upload_files(base, bc_id, fc_dir, analysis_dir, config):
                 yield (fname, os.path.basename(fname))
     for summary_file in base_glob("summary.pdf"):
         yield (summary_file, _name_with_ext(summary_file, "-summary.pdf"))
-    for bam_file in base_glob("sort-dup.bam"):
-        yield (bam_file, _name_with_ext(bam_file, ".bam"))
-    for wig_file in base_glob("sort.bigwig"):
+    for wig_file in base_glob(".bigwig"):
         yield (wig_file, _name_with_ext(wig_file, "-coverage.bigwig"))
-    # upload any recalibrated BAM files used for SNP calling
-    found_recal = False
-    for bam_file in base_glob("gatkrecal-realign-sort.bam"):
-        found_recal = True
-        yield (bam_file, _name_with_ext(bam_file, "-gatkrecal-realign.bam"))
-    if not found_recal:
-        for bam_file in base_glob("gatkrecal.bam"):
-            yield (bam_file, _name_with_ext(bam_file, "-gatkrecal.bam"))
+    # upload BAM files, preferring recalibrated and realigned files
+    found_bam = False
+    for orig_ext, new_ext in [("gatkrecal-realign-dup.bam", "-gatkrecal-realign.bam"),
+                              ("gatkrecal-realign.bam", "-gatkrecal-realign.bam"),
+                              ("gatkrecal.bam", "-gatkrecal.bam"),
+                              ("sort-dup.bam", ".bam"),
+                              ("sort.bam", ".bam")]:
+        if not found_bam:
+            for bam_file in base_glob(orig_ext):
+                yield (bam_file, _name_with_ext(bam_file, new_ext))
+                found_bam = True
     # Genotype files produced by SNP calling
-    for snp_file in base_glob("snp-filter.vcf"):
-        yield (snp_file, _name_with_ext(bam_file, "-snp-filter.vcf"))
+    found = False
+    for orig_ext, new_ext in [("variants-combined-annotated.vcf", "-variants.vcf"),
+                              ("variants-*-annotated.vcf", "-variants.vcf")]:
+        if not found:
+            for snp_file in base_glob(orig_ext):
+                yield (snp_file, _name_with_ext(bam_file, new_ext))
+                found = True
     # Effect information on SNPs
-    for snp_file in base_glob("snp-filter-effects.tsv"):
-        yield (snp_file, _name_with_ext(bam_file, "-snp-effects.tsv"))
+    for snp_file in base_glob("variants-*-effects.tsv"):
+        yield (snp_file, _name_with_ext(bam_file, "-variants-effects.tsv"))
 
 def _dir_glob(base, work_dir):
     # Allowed characters that can trail the base. This prevents picking up
@@ -168,15 +193,6 @@ def _dir_glob(base, work_dir):
         return glob.glob(os.path.join(work_dir, "%s%s*%s" % (base, trailers, ext)))
     return _safe_glob
 
-def _name_with_ext(orig_file, ext):
-    """Return a normalized filename without internal processing names.
-    """
-    base = os.path.basename(orig_file).split("-")[0]
-    for extra in ["_trim"]:
-        if base.endswith(extra):
-            base = base[:-len(extra)]
-    return "%s%s" % (base, ext)
-
 def add_run_summary_metrics(analysis_dir, galaxy_api):
     """Upload YAML file of run information to Galaxy though the NGLims API.
     """
@@ -231,7 +247,7 @@ def _folders_by_name(name, items):
                                 f['name'] == name]
 
 def move_to_storage(lane, bc_id, fc_dir, select_files, cur_galaxy_files,
-                    config, config_file):
+                    config, config_file, fname_out=None):
     """Create directory for long term storage before linking to Galaxy.
     """
     galaxy_config_file = utils.add_full_path(config["galaxy_config"],
@@ -244,22 +260,22 @@ def move_to_storage(lane, bc_id, fc_dir, select_files, cur_galaxy_files,
         raise ValueError("Galaxy config %s needs library_import_dir to be set."
                          % galaxy_config_file)
     storage_dir = _get_storage_dir(fc_dir, lane, bc_id, os.path.join(lib_import_dir,
-                                   "storage"))
+                                   "storage"), fname_out)
     existing_files = [os.path.basename(f['name']) for f in cur_galaxy_files]
     need_upload = False
     for orig_file, new_file in select_files:
-        if new_file in existing_files:
-            need_upload = False
-            break
-        else:
+        if new_file not in existing_files:
             new_file = os.path.join(storage_dir, new_file)
             if not os.path.exists(new_file):
                 shutil.copy(orig_file, new_file)
             need_upload = True
     return (storage_dir if need_upload else None)
 
-def _get_storage_dir(cur_folder, lane, bc_id, storage_base):
-    base = "%s_%s" % (lane, bc_id) if bc_id else str(lane)
+def _get_storage_dir(cur_folder, lane, bc_id, storage_base, fname_out=None):
+    if fname_out:
+        base = str(fname_out)
+    else:
+        base = "%s_%s" % (lane, bc_id) if bc_id else str(lane)
     store_dir = os.path.join(storage_base, cur_folder, base)
     utils.safe_makedir(store_dir)
     return store_dir
